@@ -15,6 +15,10 @@ import email
 from subprocess import PIPE, Popen
 from threading import Thread
 from Queue import Queue, Empty
+try:
+	import json
+except ImportError:
+	import simplejson as json
 
 pygst.require('0.10')
 gobject.threads_init()
@@ -46,6 +50,32 @@ HEIGHT = 0
 photo = None
 able_to_listen = True
 
+# Possible values:
+#	"always": always use Google Voice for speech recognition
+#	"unrecognized": use Google Voice when pocketsphinx+saera_processing can't tell what you meant
+#	"never": never use Google Voice for processing
+settings = {}
+settings['use_google_voice'] = "unrecognized"
+settings['language'] = 'english'
+lang = "en"
+lang_flag = "wiki"
+gv_api_url='http://www.google.com/speech-api/v1/recognize?lang='+lang+'-'+lang+'&client=chromium'
+SAMPLE_RATE = 16000
+try:
+	parsed_settings = json.loads(open(os.getenv('HOME')+'/.saera').read())
+	for i in parsed_settings:
+		settings[i] = parsed_settings[i]
+	__import__("sentences.sentences_"+settings['language'])
+	saera_processing.sent = sys.modules["sentences.sentences_"+settings['language']]
+except IOError:
+	open(os.getenv('HOME')+'/.saera', 'w').write(json.dumps(settings))
+
+
+try:
+	os.mkdir('/tmp/saera')
+except OSError:
+	pass
+
 class MailFolder:
 	def __init__(self, foldername, foldertype):
 		self.foldername = foldername
@@ -224,10 +254,18 @@ class Saera:
 			#pulsing = True
 			#self.pulse()
 			global able_to_listen
-			able_to_listen = False
-			os.system("aplay dingding.wav &")
-			os.system("sleep 0.5")
-			self.pipeline.set_state(gst.STATE_PLAYING)
+			if able_to_listen:
+				able_to_listen = False
+				os.system("aplay dingding.wav &")
+				os.system("sleep 0.5")
+				self.pipeline.set_state(gst.STATE_PLAYING)
+			else:
+				self.pipeline.set_state(gst.STATE_PAUSED)
+				try:
+					self.run_saera(None, 'speech-event', ' ')
+				finally:
+					os.remove('/tmp/saera/output.flac')
+					able_to_listen = True
 
 	def check_proximity_sensor(self):
 		global able_to_listen
@@ -377,8 +415,11 @@ class Saera:
 		def set_lang(widget, langs):
 			sel = [None]
 			def selection_changed(selector, seltype):
+				global settings
 				__import__("sentences.sentences_"+selector.get_current_text())
 				saera_processing.sent = sys.modules["sentences.sentences_"+selector.get_current_text()]
+				settings['language'] = selector.get_current_text()
+				open(os.getenv('HOME')+'/.saera', 'w').write(json.dumps(settings))
 				stack = hildon.WindowStack.get_default()
 				stack.pop_1()
 			lwin = hildon.StackableWindow()
@@ -425,12 +466,43 @@ class Saera:
 		else:
 			text = self.input.get_text()
 		if text:
+			print "Got input: "+text
 			self.lines.append([text, True])
 			self.input.set_text("")
 			self.darea.queue_draw()
 			result, func = saera_processing.parse_input(text+" ")
 			if func:
 				eval(func)
+			elif result.startswith("I don't understand "):
+				if settings['use_google_voice']=="unrecognized" and event=="speech-event":
+					print "Sending to Google..."
+					# os.system('wget -q -U "Mozilla/5.0" --post-file /tmp/saera/output.flac' \
+						# + ' --header="Content-Type: audio/x-flac; rate='+str(SAMPLE_RATE)+'" -O - '+gv_api_url+' > /tmp/saera/parsed.ret')
+					os.system(r'wget -q -U "Mozilla/5.0" --post-file /tmp/saera/output.flac '\
+						'--header="Content-Type: audio/x-flac; rate='+str(SAMPLE_RATE)+r'" "'\
+						+gv_api_url+'" -O /tmp/saera/parsed.ret && '+"cat /tmp/saera/parsed.ret | sed 's/.*utterance\":\"//' | " \
+						+"sed 's/\",\"confidence.*//' > /tmp/saera/result.txt")
+					print (r'wget -q -U "Mozilla/5.0" --post-file /tmp/saera/output.flac '\
+						'--header="Content-Type: audio/x-flac; rate='+str(SAMPLE_RATE)+r'" -O '\
+						'- '+gv_api_url+' > /tmp/saera/parsed.ret && '+"cat /tmp/saera/parsed.ret | sed 's/.*utterance\":\"//' | " \
+						+"sed 's/\",\"confidence.*//' > /tmp/saera/result.txt")
+					text = open('/tmp/saera/result.txt').read().strip()
+					text = text[0].upper()+text[1:].lower().replace(" and nine hundred", " N900")
+					self.lines[-1][0] = text
+					print "Got back '"+text+"'"
+				query = text.replace(" ", "_")
+				request_url = "http://"+lang_flag+".answers.com/Q/"+query
+				print "Sending request to answers.com..."
+				os.system('wget "'+request_url+'" -O /tmp/saera/answer.html')
+				answer = open('/tmp/saera/answer.html').read()
+				if 'description' in answer:
+					result = [i for i in answer.split('\n') if 'description' in i][0]
+					if "content" in result:
+						result = "".join([i for i in result.split('content="')[1].strip('"\'<>') if not i in '"<>*'])
+					else:
+						result = "I don't understand "+text+". Sorry"
+				else:
+					result = "I don't understand "+text+". Sorry."
 			global pulsing
 			global able_to_listen
 			pulsing = False
@@ -492,9 +564,16 @@ class Saera:
 
 	def init_gst(self):
 		"""Initialize the speech components"""
-		self.pipeline = gst.parse_launch('pulsesrc ! audioconvert ! audioresample '
+		self.pipeline = gst.parse_launch('pulsesrc ! audioconvert '
+										 + '! tee name = t ! audioresample '
 										 + '! vader name=vad auto-threshold=true '
-										 + '! pocketsphinx name=asr ! fakesink')
+										 # + '! pocketsphinx name=asr ! fakesink')
+										 + '! pocketsphinx name=asr t. ! audiorate ! audio/x-raw-int, rate='+str(SAMPLE_RATE)+' ! flacenc'
+										 # + '! tee name = t ! queue '
+										 # + '! vader name=vad auto-threshold=true '
+										 # # + '! pocketsphinx name=asr ! fakesink')
+										 # + '! pocketsphinx name=asr t. ! queue ! audiorate ! audio/x-raw-int, rate=48000 ! flacenc'
+										 + ' ! filesink location=/tmp/saera/output.flac')
 		asr = self.pipeline.get_by_name('asr')
 		asr.connect('partial_result', self.asr_partial_result)
 		asr.connect('result', self.asr_result)
