@@ -29,8 +29,7 @@ except:
 		ALARM_EVENT_BOOT = 8007
 
 # Strings
-import sentences.sentences_english as sent
-#import sentences.sentences_german as sent
+# import sentences.sentences_english as sent
 
 next_func = None
 location = None
@@ -100,8 +99,8 @@ def parse_input(input):
 		return where_am_i(input)
 
 	#Rather last-ditch: if we can't parse, ask Google
-	elif input.startswith("what") or input.startswith("who") or input.startswith("where") or input.startswith("how") or input.startswith("where"):
-		return search_google(input)
+	#elif input.startswith("what") or input.startswith("who") or input.startswith("where") or input.startswith("how") or input.startswith("where"):
+	#	return search_google(input)
 		
 	elif input in ("quit ", "go away "):
 		gobject.timeout_add(3000,sys.exit,0)
